var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};



















var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var toString = Object.prototype.toString;

var typeOf = (function (val) {
  var type = typeof val === 'undefined' ? 'undefined' : _typeof(val);

  if (type === 'undefined') {
    return 'undefined';
  }

  if (type === 'string' || val instanceof String) {
    return 'string';
  }
  if (type === 'number' || val instanceof Number) {
    return 'number';
  }

  if (type === 'function' || val instanceof Function) {
    return 'function';
  }

  if (typeof Array.isArray !== 'undefined' && Array.isArray(val)) {
    return 'array';
  }

  if (val instanceof Date || toString.call(val) === '[object Date]') {
    return 'date';
  }

  if ((typeof window === 'undefined' ? 'undefined' : _typeof(window)) === 'object' && typeof val.nodeType === 'number' && typeof val.nodeName === 'string') {
    return 'element';
  }

  return 'object';
});

var addLeadingZero = function addLeadingZero(num, length) {
  var str = String(num);
  while (str.length < length) {
    str = '0' + str;
  }

  return str;
};

var monthDays = function monthDays(month, year) {
  return new Date(Date.UTC(year, month + 1, 0)).getUTCDate();
};

var getOrdinalLetters = function getOrdinalLetters(num) {
  var cent = num % 100;
  if (cent >= 10 && cent <= 20) {
    return 'th';
  }

  var dec = num % 10;
  if (dec === 1) {
    return 'st';
  }
  if (dec === 2) {
    return 'nd';
  }
  if (dec === 3) {
    return 'rd';
  }
  return 'th';
};

var errorMessage = (function (_ref) {
  var functionName = _ref.functionName,
      expectedType = _ref.expectedType,
      position = _ref.position,
      messageType = _ref.messageType;

  return 'The ' + position + getOrdinalLetters(position) + ' argument to function `' + functionName + '` ' + (messageType === 'unexpected' ? 'has an unexpected type' : 'is not defined') + ', a ' + expectedType + ' is expected.';
});

var validateArguments = (function (_ref) {
  var expectedTypes = _ref.expectedTypes,
      functionName = _ref.functionName,
      args = _ref.args;

  args.forEach(function (arg, index) {
    var types = typeOf(expectedTypes[index]) === 'array' ? expectedTypes[index] : [expectedTypes[index]];

    var defaultErrorMessage = {
      functionName: functionName,
      position: index + 1,
      expectedType: types.join(' or ')
    };

    if (typeOf(arg) === 'undefined') {
      throw new TypeError(errorMessage(_extends({}, defaultErrorMessage, {
        messageType: 'notDefined'
      })));
    }

    var conditions = types.map(function (type) {
      return typeOf(arg) !== type;
    });

    if (!conditions.some(function (condition) {
      return condition === false;
    })) {
      throw new TypeError(errorMessage(_extends({}, defaultErrorMessage, {
        messageType: 'unexpected'
      })));
    }
  });
});

var flattenArray = function flattenArray(arr) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : { escapeValidation: false },
      escapeValidation = _ref.escapeValidation;

  if (!escapeValidation) {
    validateArguments({
      args: [arr],
      functionName: 'flattenArray',
      expectedTypes: ['array']
    });
  }

  return arr.reduce(function (acc, value) {
    if (typeOf(value) === 'array') {
      return acc.concat(flattenArray(value, { escapeValidation: true }));
    }

    return acc.concat(value);
  }, []);
};

function curry(fn) {
  validateArguments({
    args: [fn],
    functionName: 'curry',
    expectedTypes: ['function']
  });

  for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  if (args.length === fn.length) {
    return fn.apply(undefined, args);
  }
  return curry.bind.apply(curry, [this, fn].concat(args));
}

var cleanUpString = (function (str) {
  validateArguments({
    args: [str],
    functionName: 'cleanUpString',
    expectedTypes: ['string']
  });

  return str.replace(/(\r\n|\n|\r)/gm, '').trim();
});

var capitalize = (function (str) {
  validateArguments({
    args: [str],
    functionName: 'capitalize',
    expectedTypes: ['string']
  });

  if (cleanUpString(str) === '') {
    return str;
  }

  return str[0].toUpperCase() + str.slice(1);
});

var mapObject = function mapObject(mode) {
  return curry(function (fn, object) {
    var functionName = 'map' + capitalize(mode);
    validateArguments({
      args: [fn, object],
      functionName: functionName,
      expectedTypes: ['function', 'object']
    });

    var transformation = {
      values: function values(key) {
        return defineProperty({}, key, fn(object[key]));
      },
      keys: function keys(key) {
        return defineProperty({}, fn(key), object[key]);
      }
    };

    return Object.keys(object).map(transformation[mode]).reduce(function (previous, current) {
      return _extends({}, previous, current);
    }, {});
  });
};

var mapValues = mapObject('values');

var SECOND = 1e3;
var MINUTE = 6e4;
var HOUR = 3.6e6;
var DAY = 8.64e7;
var WEEK = 6.048e8;
var MONTH = 2.592e9;
var YEAR = 3.1536e10;

var baseDictionary = {
  YYYY: 'FullYear',
  YY: 'Year',
  MM: 'Month',
  DD: 'Date',
  HH: 'Hours',
  mm: 'Minutes',
  ss: 'Seconds'
};

var getDictionary = function getDictionary(mode) {
  return mapValues(function (value) {
    return mode + value;
  }, baseDictionary);
};

var validKeysRegex = Object.keys(baseDictionary).map(function (key, index, arr) {
  var group = '(' + key + ')';
  if (index !== arr.length - 1) {
    return group + '|';
  }
  return group;
}).join('');

var formatsRegex = new RegExp('{(' + validKeysRegex + ')}', 'g');

var validateDateArguments = function validateDateArguments(_ref) {
  var functionName = _ref.functionName,
      args = _ref.args;

  validateArguments({
    args: args,
    functionName: functionName,
    expectedTypes: ['number', 'date']
  });
};

var addMonthsWithoutValidation = function addMonthsWithoutValidation(months, date) {
  var monthsAhead = date.getMonth() + months;
  var finalDate = new Date(0);
  finalDate.setFullYear(date.getFullYear(), monthsAhead, 1);
  finalDate.setHours(date.getHours(), date.getMinutes(), date.getSeconds(), date.getMilliseconds());
  var daysInMonth = monthDays(finalDate.getMonth(), finalDate.getFullYear());
  finalDate.setDate(Math.min(daysInMonth, date.getDate()));
  return finalDate;
};

var addDateHelper = function addDateHelper(multiplicationFactor, functionName) {
  return curry(function (time, date) {
    validateDateArguments({
      functionName: functionName,
      args: [time, date]
    });

    return new Date(date.getTime() + time * multiplicationFactor);
  });
};

var addDays = addDateHelper(DAY, 'addDays');

var addHours = addDateHelper(HOUR, 'addHours');

var addMilliseconds = addDateHelper(1, 'addMilliseconds');

var addMinutes = addDateHelper(MINUTE, 'addMinutes');

var addMonths = curry(function (months, date) {
  validateDateArguments({
    functionName: 'addMonths',
    args: [months, date]
  });

  return addMonthsWithoutValidation(months, date);
});

var addSeconds = addDateHelper(SECOND, 'addSeconds');

var addWeeks = addDateHelper(WEEK, 'addWeeks');

var addYears = curry(function (years, date) {
  validateDateArguments({
    functionName: 'addYears',
    args: [years, date]
  });

  return addMonthsWithoutValidation(years * 12, date);
});

var round = (function (num) {
  validateArguments({
    args: [num],
    functionName: 'round',
    expectedTypes: ['number']
  });
  return Number(Math.sign(num) * (Math.round(Math.abs(num) + 'e2') + 'e-2'));
});

var diffDate = curry(function (firstDate, secondDate) {
  validateArguments({
    args: [firstDate, secondDate],
    functionName: 'diffDate',
    expectedTypes: ['date', 'date']
  });

  var diff = Math.abs(firstDate.getTime() - secondDate.getTime());

  var exactDiffs = {
    milliseconds: diff,
    seconds: diff / SECOND,
    minutes: diff / MINUTE,
    hours: diff / HOUR,
    days: diff / DAY,
    weeks: diff / WEEK,
    months: diff / MONTH,
    years: diff / YEAR
  };

  return mapValues(round, exactDiffs);
});

var formatDate = curry(function (format, date) {
  validateArguments({
    args: [format, date],
    functionName: 'formatDate',
    expectedTypes: ['string', 'date']
  });

  return format.replace(formatsRegex, function (fullMatch) {
    var match = fullMatch.replace(/(\{)|(\})/g, '');

    var num = match === 'YY' ? String(date.getFullYear()).substring(2, 4) : Date.prototype[getDictionary('get')[match]].call(date);

    if (match === 'MM') {
      return addLeadingZero(num + 1, 2);
    }

    return addLeadingZero(num, 2);
  });
});

var parseDate = curry(function (format, dateStr) {
  validateArguments({
    args: [format, dateStr],
    functionName: 'parseDate',
    expectedTypes: ['string', 'string']
  });

  var date = new Date(0);

  var numberOfDividers = 0;

  format.replace(formatsRegex, function (fullMatch) {
    var _ref;

    var match = fullMatch.replace(/(\{)|(\})/g, '');
    var offset = (_ref = (arguments.length <= 1 ? 0 : arguments.length - 1) - 2 + 1, arguments.length <= _ref ? undefined : arguments[_ref]) + numberOfDividers;
    var value = dateStr.substr(offset, match.length);

    if (match === 'MM') {
      value--;
    }

    if (match === 'YY' && Number(value) < 30) {
      date.setFullYear('20' + value);
    } else {
      Date.prototype[getDictionary('set')[match]].call(date, value);
    }

    numberOfDividers -= 2;
    return fullMatch;
  });

  return date;
});

var isValidDate = curry(function (format, dateStr) {
  validateArguments({
    args: [format, dateStr],
    functionName: 'isValidDate',
    expectedTypes: ['string', 'string']
  });

  var date = parseDate(format, dateStr);

  return formatDate(format, date) === dateStr;
});

var validateClassManipulationArguments = function validateClassManipulationArguments(_ref) {
  var functionName = _ref.functionName,
      args = _ref.args;

  validateArguments({
    args: args,
    functionName: functionName,
    expectedTypes: [['string', 'array'], 'element']
  });
};

var addClass = curry(function (newClass, element) {
  validateClassManipulationArguments({
    functionName: 'addClass',
    args: [newClass, element]
  });

  element.className = element.className.split(' ').filter(function (str) {
    return str.trim() !== '';
  }).concat(typeOf(newClass) === 'string' ? [newClass] : newClass).join(' ');

  return element;
});

var replaceElement = curry(function (originalElement, newElement) {
  validateArguments({
    args: [originalElement, newElement],
    functionName: 'replaceElement',
    expectedTypes: ['element', 'element']
  });

  originalElement.parentNode.replaceChild(newElement, originalElement);

  return newElement;
});

var clearEvents = (function (element) {
  validateArguments({
    args: [element],
    functionName: 'clearEvents',
    expectedTypes: ['element']
  });

  var clonedElement = element.cloneNode(true);

  if (element.parentNode) {
    replaceElement(element, clonedElement);
  }

  return clonedElement;
});

var getParents = (function (element) {
  validateArguments({
    args: [element],
    functionName: 'getParents',
    expectedTypes: ['element']
  });

  var parent = element;
  var parents = [];

  while (parent !== document.body) {
    parent = parent.parentNode;

    if (!parent) {
      break;
    }
    parents.push(parent);
  }

  return parents;
});

var hasClass = curry(function (classToCheck, element) {
  validateClassManipulationArguments({
    functionName: 'hasClass',
    args: [classToCheck, element]
  });

  var classesToCheck = typeOf(classToCheck) === 'string' ? [classToCheck] : classToCheck;

  var className = element.className.split(' ').filter(function (str) {
    return str.trim() !== '';
  });

  return classesToCheck.every(function (item) {
    return className.indexOf(item) !== -1;
  });
});

var hideElement = (function (elementsToHide) {
  validateArguments({
    args: [elementsToHide],
    functionName: 'hideElement',
    expectedTypes: [['element', 'array']]
  });

  var elements = typeOf(elementsToHide) === 'array' ? elementsToHide : [elementsToHide];

  elements.forEach(function (element) {
    element.style.display = 'none';
  });

  return elementsToHide;
});

var insertElementAfter = curry(function (referenceElement, newElement) {
  validateArguments({
    args: [referenceElement, newElement],
    functionName: 'insertElementAfter',
    expectedTypes: ['element', 'element']
  });

  referenceElement.parentNode.insertBefore(newElement, referenceElement.nextSibling);

  return newElement;
});

var insertElementBefore = curry(function (referenceElement, newElement) {
  validateArguments({
    args: [referenceElement, newElement],
    functionName: 'insertElementBefore',
    expectedTypes: ['element', 'element']
  });

  referenceElement.parentNode.insertBefore(newElement, referenceElement);

  return newElement;
});

var killElement = (function (elementsToKill) {
  validateArguments({
    args: [elementsToKill],
    functionName: 'killElement',
    expectedTypes: [['element', 'array']]
  });

  var elements = typeOf(elementsToKill) === 'array' ? elementsToKill : [elementsToKill];

  elements.forEach(function (element) {
    element.parentNode.removeChild(element);
  });
});

var removeClass = curry(function (classToRemove, element) {
  validateClassManipulationArguments({
    functionName: 'removeClass',
    args: [classToRemove, element]
  });

  element.className = element.className.split(' ').filter(function (str) {
    if (str.trim() === '') {
      return false;
    }

    if (typeOf(classToRemove) === 'string' && str === classToRemove) {
      return false;
    }

    if (typeOf(classToRemove) === 'array' && classToRemove.some(function (item) {
      return item === str;
    })) {
      return false;
    }

    return true;
  }).join(' ');

  return element;
});

var showElement = (function (elementsToShow) {
  validateArguments({
    args: [elementsToShow],
    functionName: 'showElement',
    expectedTypes: [['element', 'array']]
  });

  var elements = typeOf(elementsToShow) === 'array' ? elementsToShow : [elementsToShow];

  elements.forEach(function (element) {
    element.style.display = 'block';
  });

  return elementsToShow;
});

var toggleClass = curry(function (classToToggle, element) {
  validateClassManipulationArguments({
    functionName: 'toggleClass',
    args: [classToToggle, element]
  });

  var classesToToggle = typeOf(classToToggle) === 'string' ? [classToToggle] : classToToggle;

  classesToToggle.forEach(function (item) {
    if (hasClass(item, element)) {
      removeClass(item, element);
    } else {
      addClass(item, element);
    }
  });

  return element;
});

var compose = (function () {
  for (var _len = arguments.length, fns = Array(_len), _key = 0; _key < _len; _key++) {
    fns[_key] = arguments[_key];
  }

  validateArguments({
    args: fns,
    functionName: 'compose',
    expectedTypes: Array(fns.length).fill('function')
  });

  return function (value) {
    return fns.reduceRight(function (accumulator, current) {
      return current(accumulator);
    }, value);
  };
});

var pipe = (function () {
  for (var _len = arguments.length, fns = Array(_len), _key = 0; _key < _len; _key++) {
    fns[_key] = arguments[_key];
  }

  validateArguments({
    args: fns,
    functionName: 'pipe',
    expectedTypes: Array(fns.length).fill('function')
  });

  return function (value) {
    return fns.reduce(function (accumulator, current) {
      return current(accumulator);
    }, value);
  };
});

var throttle = curry(function (limit, fn) {
  validateArguments({
    args: [limit, fn],
    functionName: 'throttle',
    expectedTypes: ['number', 'function']
  });

  var wait = false;

  return function () {
    if (!wait) {
      fn.apply(undefined, arguments);
      wait = true;
      setTimeout(function () {
        wait = false;
      }, limit);
    }
  };
});

var mapKeys = mapObject('keys');

export { flattenArray, addDays, addHours, addMilliseconds, addMinutes, addMonths, addSeconds, addWeeks, addYears, diffDate, formatDate, isValidDate, parseDate, addClass, clearEvents, getParents, hasClass, hideElement, insertElementAfter, insertElementBefore, killElement, removeClass, replaceElement, showElement, toggleClass, compose, curry, pipe, throttle, round, mapKeys, mapValues, capitalize, cleanUpString };
